Package: seairmobility
Type: Package
Title: Mobility-Based SEAIR Epidemic Models
Version: 0.1.0
Authors@R: person(given = "Weinan",
                  family = "Wang",
                  role = c("aut", "cre"),
                  email = "ww@ou.edu")
Description: Tools for simulating, analysing, and fitting mobility-based
    SEAIR (Susceptible-Exposed-Asymptomatic-Infectious-Recovered)
    compartmental epidemic models with heterogeneous individual
    mobility. Each individual carries a fixed mobility trait that
    scales susceptibility and infectiousness through a rank-one
    kernel, extending the mobility-based compartmental framework of
    Jiang et al. (2025) <doi:10.1137/24M1691557> by adding a latent
    stage and a behavioural split between asymptomatic and symptomatic
    infectiousness. Provides a numerical solver for the underlying
    partial differential equation system, closed-form computation of
    the basic reproduction number R0 and the final epidemic size, and
    a parametric least-squares routine for recovering the mobility
    distribution from an observed aggregate symptomatic time series.
License: MIT + file LICENSE
Encoding: UTF-8
RoxygenNote: 7.3.2
Imports: deSolve (>= 1.30), stats, graphics
Suggests: testthat (>= 3.0.0)
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-25 01:30:00 UTC; cran-review
Author: Weinan Wang [aut, cre]
Maintainer: Weinan Wang <ww@ou.edu>
Repository: CRAN
Date/Publication: 2026-04-28 19:30:13 UTC
Built: R 4.4.3; ; 2026-05-01 13:11:17 UTC; unix

export(seair_params)
export(seair_init)
export(seair_solve)
export(seair_aggregate)
export(R0_seair)
export(final_size)
export(final_size_general)
export(c_effective)
export(beta_mixture_density)
export(fit_mobility)
export(plot_seair)
importFrom(deSolve, ode)
importFrom(stats, dbeta, integrate, optim, uniroot)
importFrom(graphics, lines, matplot, legend)

# seairmobility

**Mobility-Based SEAIR Epidemic Models**

`seairmobility` provides tools for simulating, analysing, and fitting
mobility-based SEAIR (Susceptible–Exposed–Asymptomatic–Infectious–
Recovered) compartmental epidemic models with heterogeneous
individual mobility.

Each individual in the population carries a fixed mobility trait
`m ∈ (0, 1)` that scales both susceptibility and infectiousness via
a rank-one kernel. The infectious period is split into an
asymptomatic stage with relative infectiousness `α` and a
symptomatic stage with mobility-reduction factor `δ`.

The package extends the mobility-based SIR framework of
Jiang, Chu, and Li (2025,
*SIAM J. Appl. Math.* 85(5), 2355–2375,
doi:10.1137/24M1691557).

## Features

- `seair_solve()` — numerical solver for the mobility-based SEAIR
  PDE system (method-of-lines discretisation + `deSolve`).
- `R0_seair()` — closed-form basic reproduction number.
- `final_size()` / `final_size_general()` — final epidemic size via
  the scalar fixed-point equation.
- `fit_mobility()` — parametric (Beta-mixture) least-squares fit of
  the mobility distribution from an observed symptomatic time series.

## Minimal example

``` r
library(seairmobility)

pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)

m    <- seq(0, 1, length.out = 101)
f    <- dbeta(m, 2, 2)
init <- seair_init(m, f, I_seed = 1e-4)

sol  <- seair_solve(init, pars, times = seq(0, 80, by = 1))
plot_seair(sol, which = c("S", "I", "R"))

R0_seair(pars, f, m_grid = m)
final_size(pars, f, m_grid = m)
```

## License

MIT

#' Effective infectiousness duration of the SEAIR chain
#'
#' Computes the scalar
#' \deqn{c \;=\; \frac{\alpha}{\kappa + \gamma_A}
#'       \;+\; \frac{\delta\,\kappa}{(\kappa + \gamma_A)\,\gamma_I},}
#' the integrated contribution of a unit of newly infected mass to
#' the force of infection, collapsing the \eqn{E \to A \to I} stage
#' chain into a single number.  The latency rate \eqn{\sigma} does
#' not appear in \eqn{c}.
#'
#' @param params A \code{"seair_params"} object.
#'
#' @return A positive scalar.
#'
#' @examples
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' c_effective(pars)
#'
#' @export
c_effective <- function(params) {
  stopifnot(inherits(params, "seair_params"))
  with(params,
       alpha / (kappa + gamma_A)
       + delta * kappa / ((kappa + gamma_A) * gamma_I))
}

#' Basic reproduction number of the mobility-based SEAIR model
#'
#' For a mobility distribution \eqn{f} on \eqn{(0,1)}, the basic
#' reproduction number at the disease-free equilibrium is
#' \deqn{\mathcal{R}_0 \;=\; \beta\,c\,\langle m^2 f \rangle,}
#' where \eqn{c} is as in \code{\link{c_effective}} and
#' \eqn{\langle m^2 f \rangle = \int_0^1 m^2 f(m)\,dm}.
#'
#' @param params A \code{"seair_params"} object.
#' @param f A function returning the density \eqn{f(m)} on \eqn{[0,1]},
#'   or a numeric vector of \eqn{f} values evaluated on \code{m_grid}.
#' @param m_grid Mobility grid (used only if \code{f} is numeric).
#'
#' @return The basic reproduction number \eqn{\mathcal{R}_0}.
#'
#' @examples
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#'
#' ## f given as a function (normalised automatically by the caller).
#' f <- function(m) 6 * m * (1 - m)    # Beta(2, 2) density on [0, 1]
#' R0_seair(pars, f)
#'
#' ## f given as values on a grid.
#' m <- seq(0, 1, length.out = 101)
#' R0_seair(pars, f(m), m_grid = m)
#'
#' @export
R0_seair <- function(params, f, m_grid = NULL) {
  stopifnot(inherits(params, "seair_params"))
  ce <- c_effective(params)
  if (is.function(f)) {
    Q2 <- stats::integrate(function(m) m^2 * f(m), 0, 1)$value
    total <- stats::integrate(f, 0, 1)$value
    if (total <= 0) stop("f integrates to zero.")
    Q2 <- Q2 / total
  } else if (is.numeric(f)) {
    if (is.null(m_grid) || length(m_grid) != length(f))
      stop("If 'f' is numeric, 'm_grid' of matching length is required.")
    if (length(m_grid) < 3 || any(!is.finite(m_grid)) ||
        any(m_grid < 0) || any(m_grid > 1) || any(diff(m_grid) <= 0) ||
        any(!is.finite(f)) || any(f < 0)) {
      stop("'m_grid' must be an increasing grid in [0, 1] and 'f' must be finite and nonnegative.")
    }
    w  <- .trap_weights(m_grid)
    total <- sum(w * f)
    if (total <= 0) stop("f integrates to zero on the grid.")
    Q2 <- sum(w * m_grid^2 * f) / total
  } else stop("'f' must be a function or a numeric vector.")
  params$beta * ce * Q2
}

#' Final epidemic size under the vanishing-seed regime
#'
#' Solves the scalar fixed-point equation
#' \deqn{M_R^\infty \;=\; \int_0^1 m f(m)\,(1 - e^{-\beta c M_R^\infty m})\,dm}
#' for the first mobility moment of the recovered compartment at the
#' end of the epidemic (with \eqn{I_0 \equiv 0}, \eqn{R_0 \equiv 0},
#' \eqn{S_0 = f}), and returns the total final epidemic size
#' \deqn{R_\infty \;=\; 1 - \int_0^1 f(m)\,e^{-\beta c M_R^\infty m}\,dm.}
#' When the basic reproduction number is less than or equal to one,
#' the function returns \eqn{R_\infty = 0}.
#'
#' @param params A \code{"seair_params"} object.
#' @param f A mobility density, either a function on \eqn{[0,1]} or a
#'   numeric vector of values on \code{m_grid}.
#' @param m_grid Optional mobility grid used when \code{f} is numeric,
#'   or to evaluate the integrals (default: uniform grid of 201 points).
#' @param tol Tolerance for \code{\link{uniroot}}.
#'
#' @return A list with \code{R_inf} (total final size),
#'   \code{MR_inf} (first moment), \code{R0} (basic reproduction
#'   number), and \code{converged} (logical).
#'
#' @examples
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' f <- function(m) 6 * m * (1 - m)
#' final_size(pars, f)
#'
#' @export
final_size <- function(params, f, m_grid = NULL, tol = 1e-10) {
  stopifnot(inherits(params, "seair_params"))
  if (is.null(m_grid)) m_grid <- seq(0, 1, length.out = 201)

  fv <- if (is.function(f)) f(m_grid) else f
  if (length(fv) != length(m_grid))
    stop("Length of f does not match m_grid.")
  if (length(m_grid) < 3 || any(!is.finite(m_grid)) ||
      any(m_grid < 0) || any(m_grid > 1) || any(diff(m_grid) <= 0) ||
      any(!is.finite(fv)) || any(fv < 0)) {
    stop("'m_grid' must be an increasing grid in [0, 1] and 'f' must be finite and nonnegative.")
  }
  w     <- .trap_weights(m_grid)
  total <- sum(w * fv)
  if (total <= 0) stop("f integrates to zero on m_grid.")
  fv    <- fv / total

  R0 <- R0_seair(params, fv, m_grid)
  if (R0 <= 1) {
    return(list(R_inf = 0, MR_inf = 0, R0 = R0, converged = TRUE))
  }

  beta_c <- params$beta * c_effective(params)
  Mf1    <- sum(w * m_grid * fv)   # upper bound on MR_inf

  Phi <- function(x) {
    sum(w * m_grid * fv * (1 - exp(-beta_c * x * m_grid))) - x
  }

  ## Phi(0) = 0 and Phi'(0) = R0 - 1 > 0, Phi(Mf1) < 0, so root in (0, Mf1].
  lo <- 1e-12 * Mf1
  hi <- Mf1
  if (Phi(lo) <= 0 || Phi(hi) >= 0) {
    return(list(R_inf = NA_real_, MR_inf = NA_real_,
                R0 = R0, converged = FALSE))
  }
  rt <- stats::uniroot(Phi, lower = lo, upper = hi, tol = tol)
  MR_inf <- rt$root
  R_inf  <- 1 - sum(w * fv * exp(-beta_c * MR_inf * m_grid))

  list(R_inf = R_inf, MR_inf = MR_inf, R0 = R0, converged = TRUE)
}

#' Final epidemic size under general proportional initial conditions
#'
#' Solves the scalar equation of Theorem 4.3 (final-size relation),
#' \deqn{M_R^\infty + \int_0^1 m S_0(m)\,
#'     \exp\!\bigl(-\beta m [ c\,(M_R^\infty - M_R^0)
#'                         + (\delta/\gamma_I - c)\,M_I^0 ]\bigr) dm
#'     \;=\; \int_0^1 m f(m)\,dm,}
#' allowing a nonzero initial symptomatic fraction
#' \eqn{I_0 = I_{\mathrm{seed}} f}, with \eqn{E_0 = A_0 = R_0 = 0}.
#'
#' @inheritParams final_size
#' @param I_seed Initial symptomatic fraction.
#'
#' @return A list with \code{R_inf}, \code{MR_inf}, \code{R0}, and
#'   \code{converged}.
#'
#' @examples
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' f <- function(m) 6 * m * (1 - m)
#' final_size_general(pars, f, I_seed = 0.001)
#'
#' @export
final_size_general <- function(params, f, m_grid = NULL,
                               I_seed = 1e-4, tol = 1e-10) {
  stopifnot(inherits(params, "seair_params"),
            is.numeric(I_seed), length(I_seed) == 1,
            I_seed >= 0, I_seed < 1)

  if (is.null(m_grid)) m_grid <- seq(0, 1, length.out = 201)
  fv <- if (is.function(f)) f(m_grid) else f
  if (length(fv) != length(m_grid))
    stop("Length of f does not match m_grid.")
  if (length(m_grid) < 3 || any(!is.finite(m_grid)) ||
      any(m_grid < 0) || any(m_grid > 1) || any(diff(m_grid) <= 0) ||
      any(!is.finite(fv)) || any(fv < 0)) {
    stop("'m_grid' must be an increasing grid in [0, 1] and 'f' must be finite and nonnegative.")
  }
  w     <- .trap_weights(m_grid)
  total <- sum(w * fv)
  if (total <= 0) stop("f integrates to zero on m_grid.")
  fv    <- fv / total

  S0 <- (1 - I_seed) * fv
  I0 <- I_seed * fv
  MR_0 <- 0
  MI_0 <- sum(w * m_grid * I0)
  Mf_1 <- sum(w * m_grid * fv)

  ce    <- c_effective(params)
  nu    <- params$delta / params$gamma_I - ce
  beta  <- params$beta
  R0    <- R0_seair(params, fv, m_grid)

  ## Vanishing-seed: I_0 = 0, R_0 = 0, S_0 = f  => reduce to final_size().
  if (I_seed == 0) return(final_size(params, fv, m_grid, tol = tol))

  Fx <- function(x) {
    expo <- -beta * m_grid * (ce * (x - MR_0) + nu * MI_0)
    x - Mf_1 + sum(w * m_grid * S0 * exp(expo))
  }

  lo <- MR_0 + MI_0          # x_ell
  hi <- Mf_1                 # x_r
  if (hi <= lo) {
    return(list(R_inf = 0, MR_inf = lo, R0 = R0, converged = TRUE))
  }
  Fl <- Fx(lo); Fh <- Fx(hi)
  if (!(is.finite(Fl) && is.finite(Fh)) || Fl >= 0 || Fh <= 0) {
    return(list(R_inf = NA_real_, MR_inf = NA_real_,
                R0 = R0, converged = FALSE))
  }
  rt <- stats::uniroot(Fx, lower = lo, upper = hi, tol = tol)
  MR_inf <- rt$root

  expo   <- -beta * m_grid * (ce * (MR_inf - MR_0) + nu * MI_0)
  R_inf  <- 1 - sum(w * S0 * exp(expo))

  list(R_inf = R_inf, MR_inf = MR_inf, R0 = R0, converged = TRUE)
}

test_that("c_effective recovers SIR formula when sigma dominates", {
  pars <- seair_params(beta = 1, sigma = 100, kappa = 0.1,
                       gamma_A = 0.1, gamma_I = 0.2,
                       alpha = 1, delta = 1)
  ce <- c_effective(pars)
  expected <- 1 / (pars$kappa + pars$gamma_A) +
    pars$kappa / ((pars$kappa + pars$gamma_A) * pars$gamma_I)
  expect_equal(ce, expected, tolerance = 1e-12)
})

test_that("R0 is independent of sigma", {
  m <- seq(0, 1, length.out = 201)
  f <- stats::dbeta(m, 2, 3)
  pars1 <- seair_params(beta = 1, sigma = 0.1, kappa = 0.2,
                        gamma_A = 0.1, gamma_I = 0.13,
                        alpha = 0.5, delta = 0.3)
  pars2 <- seair_params(beta = 1, sigma = 10,  kappa = 0.2,
                        gamma_A = 0.1, gamma_I = 0.13,
                        alpha = 0.5, delta = 0.3)
  expect_equal(R0_seair(pars1, f, m),
               R0_seair(pars2, f, m),
               tolerance = 1e-10)
})

test_that("final_size returns zero when R0 <= 1", {
  pars <- seair_params(beta = 0.01, sigma = 0.3, kappa = 0.2,
                       gamma_A = 0.5, gamma_I = 0.5,
                       alpha = 0.1, delta = 0.1)
  fs <- final_size(pars, function(m) 6 * m * (1 - m))
  expect_lte(fs$R0, 1)
  expect_equal(fs$R_inf, 0)
})

test_that("final_size returns a valid supercritical final size", {
  m <- seq(0, 1, length.out = 51)
  f <- stats::dbeta(m, 2, 2)
  pars <- seair_params(beta = 2, sigma = 0.3, kappa = 0.2,
                       gamma_A = 0.1, gamma_I = 0.2,
                       alpha = 0.5, delta = 0.5)
  fs <- final_size(pars, f, m_grid = m)
  expect_true(fs$converged)
  expect_gt(fs$R0, 1)
  expect_gt(fs$R_inf, 0)
  expect_lt(fs$R_inf, 1)
})


test_that("final_size_general returns zero when R0 <= 1 and I_seed = 0", {
  pars <- seair_params(beta = 0.01, sigma = 0.3, kappa = 0.2,
                       gamma_A = 0.5, gamma_I = 0.5,
                       alpha = 0.1, delta = 0.1)
  fs <- final_size_general(pars,
                           function(m) 6 * m * (1 - m),
                           I_seed = 0)
  expect_lte(fs$R0, 1)
  expect_equal(fs$R_inf, 0)
})

#' seairmobility: Mobility-Based SEAIR Epidemic Models
#'
#' The \pkg{seairmobility} package provides tools for simulating and
#' analysing mobility-based SEAIR compartmental epidemic models.
#' Each individual in the population carries a fixed mobility trait
#' \eqn{m \in (0,1)} that scales both their susceptibility and their
#' infectiousness through a rank-one transmission kernel.  The
#' infectious period is split into an asymptomatic stage with
#' relative infectiousness \eqn{\alpha} and a symptomatic stage with
#' mobility-reduction factor \eqn{\delta}.
#'
#' The main functions are
#' \describe{
#'   \item{\code{\link{seair_params}}}{Build a parameter list.}
#'   \item{\code{\link{seair_init}}}{Build initial conditions.}
#'   \item{\code{\link{seair_solve}}}{Solve the SEAIR system.}
#'   \item{\code{\link{seair_aggregate}}}{Aggregate solution over
#'     mobility.}
#'   \item{\code{\link{R0_seair}}}{Basic reproduction number.}
#'   \item{\code{\link{final_size}}}{Final epidemic size under vanishing
#'     seed.}
#'   \item{\code{\link{fit_mobility}}}{Parametric least-squares fit of
#'     the mobility distribution from an observed symptomatic time
#'     series.}
#' }
#'
#' @references
#' Jiang, N., Chu, W., and Li, Y. (2025).
#' Modeling, inference, and prediction in mobility-based
#' compartmental models for epidemiology.
#' \emph{SIAM Journal on Applied Mathematics}, 85(5), 2355--2375.
#' \doi{10.1137/24M1691557}.
#'
#' @name seairmobility-package
#' @aliases seairmobility
"_PACKAGE"

#' Plot aggregate compartment trajectories
#'
#' Convenience plot of the five aggregate compartment trajectories
#' produced by a SEAIR solution.
#'
#' @param sol A \code{"seair_solution"} object.
#' @param which Character vector of compartments to plot; any subset
#'   of \code{c("S","E","A","I","R")}.
#' @param ... Additional graphical parameters passed to
#'   \code{\link[graphics]{matplot}}.
#'
#' @return Invisibly returns the aggregate data frame.
#'
#' @examples
#' m <- seq(0, 1, length.out = 31)
#' f <- dbeta(m, 2, 2)
#' init <- seair_init(m, f, I_seed = 1e-4)
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))
#' plot_seair(sol, which = c("S","I","R"))
#'
#' @export
plot_seair <- function(sol,
                       which = c("S", "E", "A", "I", "R"),
                       ...) {
  stopifnot(inherits(sol, "seair_solution"))
  which <- match.arg(which, c("S", "E", "A", "I", "R"),
                     several.ok = TRUE)
  agg <- seair_aggregate(sol)
  Y <- as.matrix(agg[, which, drop = FALSE])
  graphics::matplot(agg$time, Y, type = "l", lty = 1,
                    xlab = "time", ylab = "population fraction",
                    ...)
  graphics::legend("topright", legend = which, lty = 1,
                   col = seq_along(which), bty = "n")
  invisible(agg)
}

#' Solve the mobility-based SEAIR system
#'
#' Discretises the mobility-based SEAIR system on the user-supplied
#' mobility grid (using the trapezoidal rule for the non-local
#' integral \eqn{\mathcal{K}}) and integrates the resulting coupled
#' ODE system over \code{times} with \pkg{deSolve}.
#'
#' @param init A \code{"seair_init"} object from \code{\link{seair_init}}.
#' @param params A \code{"seair_params"} object from
#'   \code{\link{seair_params}}.
#' @param times Numeric vector of times at which the solution is
#'   reported (must be increasing and include \code{0}).
#' @param method Integration method passed to \code{\link[deSolve]{ode}}
#'   (default \code{"lsoda"}).
#' @param ... Additional arguments passed to \code{\link[deSolve]{ode}}.
#'
#' @return A list of class \code{"seair_solution"} with entries
#'   \code{times}, \code{m_grid}, \code{S}, \code{E}, \code{A},
#'   \code{I}, \code{R} (each a matrix of dimension
#'   \code{length(times)} by \code{length(m_grid)}),
#'   together with the input \code{params} and \code{init}.
#'
#' @examples
#' m <- seq(0, 1, length.out = 31)
#' f <- dbeta(m, 2, 2)
#' init <- seair_init(m, f, I_seed = 1e-4)
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))
#'
#' @export
seair_solve <- function(init, params, times,
                        method = "lsoda", ...) {
  stopifnot(inherits(init, "seair_init"),
            inherits(params, "seair_params"),
            is.numeric(times), length(times) >= 2,
            all(is.finite(times)), all(diff(times) > 0), times[1] == 0)

  m  <- init$m_grid
  n  <- length(m)
  w  <- .trap_weights(m)   # quadrature weights for the trapezoidal rule

  beta    <- params$beta
  sigma   <- params$sigma
  kappa   <- params$kappa
  gamma_A <- params$gamma_A
  gamma_I <- params$gamma_I
  alpha   <- params$alpha
  delta   <- params$delta

  ## State vector layout: [S(1..n), E(1..n), A(1..n), I(1..n), R(1..n)]
  y0 <- c(init$S, init$E, init$A, init$I, init$R)

  rhs <- function(t, y, parms) {
    S <- y[1:n]
    E <- y[(n + 1):(2 * n)]
    A <- y[(2 * n + 1):(3 * n)]
    I <- y[(3 * n + 1):(4 * n)]
    R <- y[(4 * n + 1):(5 * n)]

    K <- sum(w * m * (alpha * A + delta * I))

    dS <- -beta * m * S * K
    dE <-  beta * m * S * K - sigma * E
    dA <-  sigma * E - (kappa + gamma_A) * A
    dI <-  kappa * A - gamma_I * I
    dR <-  gamma_A * A + gamma_I * I

    list(c(dS, dE, dA, dI, dR))
  }

  out <- deSolve::ode(y = y0, times = times, func = rhs,
                      parms = NULL, method = method, ...)

  mat <- unclass(out)
  tt  <- mat[, 1]
  S   <- mat[, 1 + (1:n),           drop = FALSE]
  E   <- mat[, 1 + ( (n + 1):(2 * n) ),     drop = FALSE]
  A   <- mat[, 1 + ( (2 * n + 1):(3 * n) ), drop = FALSE]
  I   <- mat[, 1 + ( (3 * n + 1):(4 * n) ), drop = FALSE]
  R   <- mat[, 1 + ( (4 * n + 1):(5 * n) ), drop = FALSE]

  sol <- list(times = tt, m_grid = m,
              S = S, E = E, A = A, I = I, R = R,
              params = params, init = init)
  class(sol) <- "seair_solution"
  sol
}

#' Aggregate a SEAIR solution over mobility
#'
#' Computes the aggregate time series
#' \eqn{\int_0^1 S(m,t)\,dm}, \eqn{\int_0^1 E(m,t)\,dm}, etc., using
#' the trapezoidal rule on the solver's mobility grid.
#'
#' @param sol A \code{"seair_solution"} object returned by
#'   \code{\link{seair_solve}}.
#'
#' @return A data frame with columns \code{time}, \code{S}, \code{E},
#'   \code{A}, \code{I}, \code{R} (aggregate ratios).
#'
#' @examples
#' m <- seq(0, 1, length.out = 31)
#' f <- dbeta(m, 2, 2)
#' init <- seair_init(m, f, I_seed = 1e-4)
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))
#' agg <- seair_aggregate(sol)
#' head(agg)
#'
#' @export
seair_aggregate <- function(sol) {
  stopifnot(inherits(sol, "seair_solution"))
  w <- .trap_weights(sol$m_grid)
  data.frame(
    time = sol$times,
    S = as.numeric(sol$S %*% w),
    E = as.numeric(sol$E %*% w),
    A = as.numeric(sol$A %*% w),
    I = as.numeric(sol$I %*% w),
    R = as.numeric(sol$R %*% w)
  )
}

## Trapezoidal quadrature weights on a (possibly non-uniform) grid.
.trap_weights <- function(x) {
  n <- length(x)
  w <- numeric(n)
  w[1]     <- (x[2] - x[1]) / 2
  w[n]     <- (x[n] - x[n - 1]) / 2
  if (n > 2) {
    w[2:(n - 1)] <- (x[3:n] - x[1:(n - 2)]) / 2
  }
  w
}

test_that("fit_mobility returns a finite deterministic fit", {
  pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                       gamma_A = 0.1, gamma_I = 0.13,
                       alpha = 0.5, delta = 0.3)
  m     <- seq(0, 1, length.out = 21)
  ftrue <- stats::dbeta(m, 3, 3)
  init  <- seair_init(m, ftrue, I_seed = 1e-4)
  times <- seq(0, 20, by = 5)
  sol   <- seair_solve(init, pars, times = times)
  I_obs <- seair_aggregate(sol)$I

  fit <- fit_mobility(times, I_obs, pars,
                      K = 1, m_grid = m,
                      start = log(c(3, 3)),
                      n_restarts = 0,
                      control = list(maxit = 20))

  expect_true(is.finite(fit$loss))
  expect_lt(fit$loss, 1e-10)
  expect_length(fit$f_hat, length(m))
})

test_that("seair_solve conserves total mass", {
  m <- seq(0, 1, length.out = 41)
  f <- stats::dbeta(m, 2, 2)
  init <- seair_init(m, f, I_seed = 1e-3)
  pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                       gamma_A = 0.1, gamma_I = 0.13,
                       alpha = 0.6, delta = 0.4)
  sol <- seair_solve(init, pars, times = seq(0, 50, by = 5))
  agg <- seair_aggregate(sol)
  totals <- rowSums(agg[, c("S", "E", "A", "I", "R")])
  expect_true(all(abs(totals - 1) < 1e-6))
})

test_that("seair_solve agrees with baseline SIR in the degenerate limit", {
  ## When alpha = delta = 1 and kappa is small, the asymptomatic stage
  ## dominates and the model is close to SIR-like dynamics; here we
  ## only check that trajectories stay non-negative and bounded.
  m <- seq(0, 1, length.out = 41)
  f <- stats::dbeta(m, 5, 5)
  init <- seair_init(m, f, I_seed = 1e-4)
  pars <- seair_params(beta = 1.0, sigma = 1.0, kappa = 1e-3,
                       gamma_A = 0.15, gamma_I = 0.15,
                       alpha = 1, delta = 1)
  sol <- seair_solve(init, pars, times = seq(0, 60, by = 2))
  expect_true(all(sol$S >= -1e-8))
  expect_true(all(sol$I >= -1e-8))
  expect_true(all(sol$R >= -1e-8))
  expect_true(max(sol$I) <= 1 + 1e-6)
})

#' Beta-mixture density on [0, 1]
#'
#' Evaluates a weighted mixture of Beta densities
#' \deqn{f(m) \;=\; \sum_{k=1}^{K} w_k\,\mathrm{Beta}(m; a_k, b_k),}
#' renormalised on \eqn{[0,1]}.  Weights are coerced to be nonnegative
#' and to sum to one.
#'
#' @param m Numeric vector of evaluation points in \eqn{[0,1]}.
#' @param weights Numeric vector of mixture weights (length \eqn{K}).
#' @param shape1 Numeric vector of Beta \eqn{a_k} parameters.
#' @param shape2 Numeric vector of Beta \eqn{b_k} parameters.
#'
#' @return Numeric vector of densities at \code{m}.
#'
#' @examples
#' m <- seq(0, 1, length.out = 101)
#' beta_mixture_density(m,
#'                      weights = c(0.5, 0.3, 0.2),
#'                      shape1  = c(2, 5, 9),
#'                      shape2  = c(8, 3, 2))
#'
#' @export
beta_mixture_density <- function(m, weights, shape1, shape2) {
  K <- length(weights)
  stopifnot(is.numeric(m),
            all(is.finite(m)), all(m >= 0), all(m <= 1),
            is.numeric(weights), K >= 1,
            is.numeric(shape1), is.numeric(shape2),
            length(shape1) == K, length(shape2) == K,
            all(is.finite(weights)),
            all(is.finite(shape1)), all(is.finite(shape2)),
            all(shape1 > 0), all(shape2 > 0))
  w <- pmax(weights, 0)
  if (sum(w) <= 0) stop("All mixture weights are zero.")
  w <- w / sum(w)
  out <- numeric(length(m))
  for (k in seq_len(K)) {
    out <- out + w[k] * stats::dbeta(m, shape1[k], shape2[k])
  }
  out
}

#' Fit a mobility distribution to an observed symptomatic time series
#'
#' Fits a Beta-mixture mobility distribution \eqn{f} by minimising the
#' sum of squared errors between an observed aggregate symptomatic
#' time series \eqn{\langle I \rangle(t)} and the simulated output of
#' the mobility-based SEAIR model at the same times.  All transmission
#' and stage parameters are held fixed.
#'
#' The unconstrained optimisation parameter is, for each component
#' \eqn{k = 1, \ldots, K}, a triple
#' \eqn{(\log a_k, \log b_k, \log \tilde{w}_k)} with mixture weights
#' recovered as \eqn{w_k = \tilde{w}_k / \sum_j \tilde{w}_j}.  The
#' first weight's log is fixed to 0 to remove the scale ambiguity,
#' so the parameter vector has length \eqn{3 K - 1}.
#'
#' @param times Numeric vector of observation times (starting at 0).
#' @param I_obs Observed aggregate symptomatic fraction at each time.
#' @param params A \code{"seair_params"} object holding fixed
#'   transmission and stage parameters.
#' @param K Number of Beta-mixture components (default 3).
#' @param m_grid Mobility grid for the solver.
#' @param I_seed Initial symptomatic fraction.
#' @param start Optional user-supplied starting parameter vector of
#'   length \eqn{3K - 1}.
#' @param n_restarts Number of random restarts (default 5).
#' @param control List of \code{\link[stats]{optim}} control options.
#' @param verbose Logical; if \code{TRUE}, print progress.
#'
#' @return A list with \code{weights}, \code{shape1}, \code{shape2}
#'   (the fitted Beta-mixture parameters), \code{f_hat} (density on
#'   \code{m_grid}), \code{m_grid}, \code{loss} (final SSE),
#'   \code{I_fit} (simulated aggregate symptomatic time series at
#'   \code{times}), and \code{optim_result}.
#'
#' @examples
#' pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'                      gamma_A = 0.1, gamma_I = 0.13,
#'                      alpha = 0.5, delta = 0.3)
#' m <- seq(0, 1, length.out = 21)
#' f_true <- dbeta(m, 3, 3)
#' init <- seair_init(m, f_true, I_seed = 1e-4)
#' times <- seq(0, 20, by = 5)
#' sol <- seair_solve(init, pars, times)
#' I_obs <- seair_aggregate(sol)$I
#'
#' fit <- fit_mobility(times, I_obs, pars,
#'                     K = 1, m_grid = m,
#'                     start = log(c(3, 3)),
#'                     n_restarts = 0,
#'                     control = list(maxit = 20))
#' fit$loss
#'
#' @export
fit_mobility <- function(times, I_obs, params,
                         K = 3,
                         m_grid = seq(0, 1, length.out = 51),
                         I_seed = 1e-4,
                         start = NULL,
                         n_restarts = 5,
                         control = list(maxit = 500),
                         verbose = FALSE) {
  stopifnot(is.numeric(times), is.numeric(I_obs),
            length(times) == length(I_obs), length(times) >= 2,
            all(is.finite(times)), all(diff(times) > 0), times[1] == 0,
            all(is.finite(I_obs)), all(I_obs >= 0),
            inherits(params, "seair_params"),
            is.numeric(K), length(K) == 1, K >= 1, K == as.integer(K),
            is.numeric(m_grid), length(m_grid) >= 3,
            all(is.finite(m_grid)), all(m_grid >= 0), all(m_grid <= 1),
            all(diff(m_grid) > 0),
            is.numeric(I_seed), length(I_seed) == 1,
            is.finite(I_seed), I_seed >= 0, I_seed < 1,
            is.numeric(n_restarts), length(n_restarts) == 1,
            n_restarts >= 0, n_restarts == as.integer(n_restarts),
            is.list(control),
            is.logical(verbose), length(verbose) == 1, !is.na(verbose))
  K <- as.integer(K)
  n_restarts <- as.integer(n_restarts)

  n_par <- 3 * K - 1
  if (is.null(start) && n_restarts == 0) {
    stop("At least one of 'start' or a positive 'n_restarts' is required.")
  }

  unpack <- function(theta) {
    log_a <- theta[1:K]
    log_b <- theta[(K + 1):(2 * K)]
    if (K == 1) {
      w_tilde <- 1
    } else {
      w_tilde <- c(1, exp(theta[(2 * K + 1):(3 * K - 1)]))
    }
    list(shape1  = exp(log_a),
         shape2  = exp(log_b),
         weights = w_tilde / sum(w_tilde))
  }

  obj <- function(theta) {
    pars_mix <- tryCatch(unpack(theta), error = function(e) NULL)
    if (is.null(pars_mix)) return(.Machine$double.xmax)
    f_vals <- tryCatch(
      beta_mixture_density(m_grid,
                           weights = pars_mix$weights,
                           shape1  = pars_mix$shape1,
                           shape2  = pars_mix$shape2),
      error = function(e) NULL)
    if (is.null(f_vals) || any(!is.finite(f_vals))) {
      return(.Machine$double.xmax)
    }
    init <- seair_init(m_grid, f_vals, I_seed = I_seed)
    sol  <- tryCatch(seair_solve(init, params, times = times),
                     error = function(e) NULL)
    if (is.null(sol)) return(.Machine$double.xmax)
    I_sim <- seair_aggregate(sol)$I
    sum((I_sim - I_obs)^2)
  }

  random_start <- function() {
    log_a <- log(stats::runif(K, 1, 10))
    log_b <- log(stats::runif(K, 1, 10))
    if (K == 1) return(c(log_a, log_b))
    c(log_a, log_b, stats::rnorm(K - 1, sd = 0.5))
  }

  best <- list(par = NULL, value = Inf, convergence = NA_integer_)

  ## User-supplied start, then random restarts.
  starts <- list()
  if (!is.null(start)) {
    if (!is.numeric(start) || any(!is.finite(start)))
      stop("'start' must be a finite numeric vector.")
    if (length(start) != n_par)
      stop(sprintf("'start' must have length %d (got %d).",
                   n_par, length(start)))
    starts[[1]] <- start
  }
  for (r in seq_len(n_restarts)) {
    starts[[length(starts) + 1]] <- random_start()
  }

  for (i in seq_along(starts)) {
    if (verbose) message("restart ", i, " of ", length(starts))
    res <- tryCatch(
      stats::optim(starts[[i]], obj, method = "Nelder-Mead",
                   control = control),
      error = function(e) NULL)
    if (!is.null(res) && is.finite(res$value) && res$value < best$value) {
      best <- res
    }
  }

  if (!is.finite(best$value)) {
    stop("Optimisation failed at all starts; try increasing 'n_restarts'.")
  }

  pars_mix <- unpack(best$par)
  f_hat    <- beta_mixture_density(m_grid,
                                   weights = pars_mix$weights,
                                   shape1  = pars_mix$shape1,
                                   shape2  = pars_mix$shape2)
  init <- seair_init(m_grid, f_hat, I_seed = I_seed)
  I_fit <- seair_aggregate(seair_solve(init, params, times))$I

  list(weights      = pars_mix$weights,
       shape1       = pars_mix$shape1,
       shape2       = pars_mix$shape2,
       f_hat        = f_hat,
       m_grid       = m_grid,
       loss         = best$value,
       I_fit        = I_fit,
       optim_result = best)
}

#' Build a parameter list for the mobility-based SEAIR model
#'
#' Constructs and validates the list of transmission and stage-progression
#' parameters used by the mobility-based SEAIR model
#' \eqn{\partial_t S = -\beta m S \mathcal{K}},
#' \eqn{\partial_t E = \beta m S \mathcal{K} - \sigma E},
#' \eqn{\partial_t A = \sigma E - (\kappa + \gamma_A) A},
#' \eqn{\partial_t I = \kappa A - \gamma_I I},
#' \eqn{\partial_t R = \gamma_A A + \gamma_I I},
#' where
#' \eqn{\mathcal{K}(A,I) = \int_0^1 \bar{m}\,(\alpha A(\bar{m}) + \delta I(\bar{m}))\,d\bar{m}}.
#'
#' @param beta Transmission rate \eqn{\beta > 0}.
#' @param sigma Rate \eqn{\sigma > 0} of progression from latent (E) to
#'   asymptomatic (A).
#' @param kappa Symptom-onset rate \eqn{\kappa > 0} (A to I).
#' @param gamma_A Recovery rate \eqn{\gamma_A > 0} from the asymptomatic
#'   compartment.
#' @param gamma_I Recovery rate \eqn{\gamma_I > 0} from the symptomatic
#'   compartment.
#' @param alpha Relative infectiousness \eqn{\alpha \in [0,1]} of the
#'   asymptomatic stage.
#' @param delta Mobility-reduction factor \eqn{\delta \in (0,1]} for the
#'   symptomatic stage.
#'
#' @return A list of class \code{"seair_params"} containing the checked
#'   parameters.
#'
#' @examples
#' seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
#'              gamma_A = 0.1, gamma_I = 0.13,
#'              alpha = 0.5, delta = 0.3)
#'
#' @export
seair_params <- function(beta, sigma, kappa, gamma_A, gamma_I,
                         alpha = 1, delta = 1) {
  stopifnot(
    is.numeric(beta),    length(beta)    == 1, beta    > 0,
    is.numeric(sigma),   length(sigma)   == 1, sigma   > 0,
    is.numeric(kappa),   length(kappa)   == 1, kappa   > 0,
    is.numeric(gamma_A), length(gamma_A) == 1, gamma_A > 0,
    is.numeric(gamma_I), length(gamma_I) == 1, gamma_I > 0,
    is.numeric(alpha),   length(alpha)   == 1, alpha >= 0, alpha <= 1,
    is.numeric(delta),   length(delta)   == 1, delta >  0, delta <= 1
  )
  out <- list(beta = beta, sigma = sigma, kappa = kappa,
              gamma_A = gamma_A, gamma_I = gamma_I,
              alpha = alpha, delta = delta)
  class(out) <- "seair_params"
  out
}

#' Build initial conditions for the SEAIR system
#'
#' Constructs initial profiles \eqn{(S_0, E_0, A_0, I_0, R_0)} on the
#' mobility grid from a user-supplied mobility density \eqn{f(m)} and a
#' seed fraction \eqn{I_{\mathrm{seed}}} placed in the symptomatic
#' compartment.  The initial condition is "proportional":
#' \eqn{S_0 = (1 - I_{\mathrm{seed}}) f},
#' \eqn{I_0 = I_{\mathrm{seed}} f},
#' \eqn{E_0 = A_0 = R_0 = 0}.
#'
#' @param m_grid Numeric vector of mobility grid points in \eqn{[0,1]}.
#' @param f_vals Numeric vector of densities \eqn{f(m)} evaluated at
#'   \code{m_grid}.  It is renormalised internally so that the
#'   trapezoidal integral over \code{m_grid} equals 1.
#' @param I_seed Non-negative fraction of the population seeded in the
#'   symptomatic compartment (default \code{1e-4}).
#'
#' @return A list of class \code{"seair_init"} with entries
#'   \code{m_grid}, \code{f}, \code{S}, \code{E}, \code{A}, \code{I},
#'   \code{R}, and \code{I_seed}.
#'
#' @examples
#' m <- seq(0, 1, length.out = 101)
#' f <- dbeta(m, 2, 2)
#' seair_init(m, f, I_seed = 1e-4)
#'
#' @export
seair_init <- function(m_grid, f_vals, I_seed = 1e-4) {
  stopifnot(is.numeric(m_grid), is.numeric(f_vals),
            length(m_grid) == length(f_vals),
            length(m_grid) >= 3,
            all(is.finite(m_grid)), all(is.finite(f_vals)),
            all(m_grid >= 0), all(m_grid <= 1), all(diff(m_grid) > 0),
            all(f_vals >= 0),
            is.numeric(I_seed), length(I_seed) == 1,
            I_seed >= 0, I_seed < 1)

  total <- .trap(m_grid, f_vals)
  if (total <= 0) stop("f_vals integrates to zero; check your density.")
  f <- f_vals / total

  S <- (1 - I_seed) * f
  I <- I_seed       * f
  E <- A <- R <- rep(0, length(m_grid))

  out <- list(m_grid = m_grid, f = f,
              S = S, E = E, A = A, I = I, R = R,
              I_seed = I_seed)
  class(out) <- "seair_init"
  out
}

## Trapezoidal rule on a (possibly non-uniform) grid.
.trap <- function(x, y) {
  n <- length(x)
  if (n < 2) return(0)
  sum((x[-1] - x[-n]) * (y[-1] + y[-n])) / 2
}

test_that("seair_params validates inputs", {
  expect_s3_class(
    seair_params(1, 0.3, 0.2, 0.1, 0.13, 0.5, 0.3),
    "seair_params")
  expect_error(seair_params(-1, 0.3, 0.2, 0.1, 0.13))
  expect_error(seair_params(1, 0.3, 0.2, 0.1, 0.13, alpha = 2))
  expect_error(seair_params(1, 0.3, 0.2, 0.1, 0.13, delta = 0))
})

test_that("seair_init normalises f and stores grid", {
  m <- seq(0, 1, length.out = 101)
  f <- rep(1, length(m))    # uniform, unnormalised
  init <- seair_init(m, f, I_seed = 0.01)
  w <- (function(x) {
    n <- length(x)
    w <- numeric(n)
    w[1] <- (x[2] - x[1]) / 2
    w[n] <- (x[n] - x[n - 1]) / 2
    if (n > 2) w[2:(n - 1)] <- (x[3:n] - x[1:(n - 2)]) / 2
    w
  })(m)
  expect_equal(sum(w * init$f), 1, tolerance = 1e-10)
  expect_equal(sum(w * init$S), 1 - 0.01, tolerance = 1e-10)
  expect_equal(sum(w * init$I), 0.01,     tolerance = 1e-10)
})

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plotting.R
\name{plot_seair}
\alias{plot_seair}
\title{Plot aggregate compartment trajectories}
\usage{
plot_seair(sol, which = c("S", "E", "A", "I", "R"), ...)
}
\arguments{
\item{sol}{A \code{"seair_solution"} object.}

\item{which}{Character vector of compartments to plot; any subset
of \code{c("S","E","A","I","R")}.}

\item{...}{Additional graphical parameters passed to
\code{\link[graphics]{matplot}}.}
}
\value{
Invisibly returns the aggregate data frame.
}
\description{
Convenience plot of the five aggregate compartment trajectories
produced by a SEAIR solution.
}
\examples{
m <- seq(0, 1, length.out = 31)
f <- dbeta(m, 2, 2)
init <- seair_init(m, f, I_seed = 1e-4)
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))
plot_seair(sol, which = c("S","I","R"))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{beta_mixture_density}
\alias{beta_mixture_density}
\title{Beta-mixture density on [0, 1]}
\usage{
beta_mixture_density(m, weights, shape1, shape2)
}
\arguments{
\item{m}{Numeric vector of finite evaluation points in \eqn{[0,1]}.}

\item{weights}{Numeric vector of mixture weights (length \eqn{K}).}

\item{shape1}{Numeric vector of Beta \eqn{a_k} parameters.}

\item{shape2}{Numeric vector of Beta \eqn{b_k} parameters.}
}
\value{
Numeric vector of densities at \code{m}.
}
\description{
Evaluates a weighted mixture of Beta densities
\deqn{f(m) \;=\; \sum_{k=1}^{K} w_k\,\mathrm{Beta}(m; a_k, b_k),}
renormalised on \eqn{[0,1]}.  Weights are coerced to be nonnegative
and to sum to one.
}
\examples{
m <- seq(0, 1, length.out = 101)
beta_mixture_density(m,
                     weights = c(0.5, 0.3, 0.2),
                     shape1  = c(2, 5, 9),
                     shape2  = c(8, 3, 2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epi_quantities.R
\name{R0_seair}
\alias{R0_seair}
\title{Basic reproduction number of the mobility-based SEAIR model}
\usage{
R0_seair(params, f, m_grid = NULL)
}
\arguments{
\item{params}{A \code{"seair_params"} object.}

\item{f}{A function returning the density \eqn{f(m)} on \eqn{[0,1]},
or a numeric vector of \eqn{f} values evaluated on \code{m_grid}.}

\item{m_grid}{Mobility grid (used only if \code{f} is numeric).}
}
\value{
The basic reproduction number \eqn{\mathcal{R}_0}.
}
\description{
For a mobility distribution \eqn{f} on \eqn{(0,1)}, the basic
reproduction number at the disease-free equilibrium is
\deqn{\mathcal{R}_0 \;=\; \beta\,c\,\langle m^2 f \rangle,}
where \eqn{c} is as in \code{\link{c_effective}} and
\eqn{\langle m^2 f \rangle = \int_0^1 m^2 f(m)\,dm}.
}
\examples{
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)

## f given as a function (normalised automatically by the caller).
f <- function(m) 6 * m * (1 - m)    # Beta(2, 2) density on [0, 1]
R0_seair(pars, f)

## f given as values on a grid.
m <- seq(0, 1, length.out = 101)
R0_seair(pars, f(m), m_grid = m)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/seairmobility-package.R
\docType{package}
\name{seairmobility-package}
\alias{seairmobility-package}
\alias{seairmobility}
\title{seairmobility: Mobility-Based SEAIR Epidemic Models}
\description{
The \pkg{seairmobility} package provides tools for simulating and
analysing mobility-based SEAIR compartmental epidemic models.
Each individual in the population carries a fixed mobility trait
\eqn{m \in (0,1)} that scales both their susceptibility and their
infectiousness through a rank-one transmission kernel.  The
infectious period is split into an asymptomatic stage with
relative infectiousness \eqn{\alpha} and a symptomatic stage with
mobility-reduction factor \eqn{\delta}.
}
\details{
The main functions are
\describe{
  \item{\code{\link{seair_params}}}{Build a parameter list.}
  \item{\code{\link{seair_init}}}{Build initial conditions.}
  \item{\code{\link{seair_solve}}}{Solve the SEAIR system.}
  \item{\code{\link{seair_aggregate}}}{Aggregate solution over
    mobility.}
  \item{\code{\link{R0_seair}}}{Basic reproduction number.}
  \item{\code{\link{final_size}}}{Final epidemic size under vanishing
    seed.}
  \item{\code{\link{fit_mobility}}}{Parametric least-squares fit of
    the mobility distribution from an observed symptomatic time
    series.}
}
}
\references{
Jiang, N., Chu, W., and Li, Y. (2025).
Modeling, inference, and prediction in mobility-based
compartmental models for epidemiology.
\emph{SIAM Journal on Applied Mathematics}, 85(5), 2355--2375.
\doi{10.1137/24M1691557}.
}
\author{
\strong{Maintainer}: Weinan Wang \email{ww@ou.edu}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epi_quantities.R
\name{c_effective}
\alias{c_effective}
\title{Effective infectiousness duration of the SEAIR chain}
\usage{
c_effective(params)
}
\arguments{
\item{params}{A \code{"seair_params"} object.}
}
\value{
A positive scalar.
}
\description{
Computes the scalar
\deqn{c \;=\; \frac{\alpha}{\kappa + \gamma_A}
      \;+\; \frac{\delta\,\kappa}{(\kappa + \gamma_A)\,\gamma_I},}
the integrated contribution of a unit of newly infected mass to
the force of infection, collapsing the \eqn{E \to A \to I} stage
chain into a single number.  The latency rate \eqn{\sigma} does
not appear in \eqn{c}.
}
\examples{
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
c_effective(pars)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/params.R
\name{seair_init}
\alias{seair_init}
\title{Build initial conditions for the SEAIR system}
\usage{
seair_init(m_grid, f_vals, I_seed = 1e-04)
}
\arguments{
\item{m_grid}{Numeric vector of mobility grid points in \eqn{[0,1]}.}

\item{f_vals}{Numeric vector of densities \eqn{f(m)} evaluated at
\code{m_grid}.  It is renormalised internally so that the
trapezoidal integral over \code{m_grid} equals 1.}

\item{I_seed}{Non-negative fraction of the population seeded in the
symptomatic compartment (default \code{1e-4}).}
}
\value{
A list of class \code{"seair_init"} with entries
  \code{m_grid}, \code{f}, \code{S}, \code{E}, \code{A}, \code{I},
  \code{R}, and \code{I_seed}.
}
\description{
Constructs initial profiles \eqn{(S_0, E_0, A_0, I_0, R_0)} on the
mobility grid from a user-supplied mobility density \eqn{f(m)} and a
seed fraction \eqn{I_{\mathrm{seed}}} placed in the symptomatic
compartment.  The initial condition is "proportional":
\eqn{S_0 = (1 - I_{\mathrm{seed}}) f},
\eqn{I_0 = I_{\mathrm{seed}} f},
\eqn{E_0 = A_0 = R_0 = 0}.
}
\examples{
m <- seq(0, 1, length.out = 101)
f <- dbeta(m, 2, 2)
seair_init(m, f, I_seed = 1e-4)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epi_quantities.R
\name{final_size}
\alias{final_size}
\title{Final epidemic size under the vanishing-seed regime}
\usage{
final_size(params, f, m_grid = NULL, tol = 1e-10)
}
\arguments{
\item{params}{A \code{"seair_params"} object.}

\item{f}{A mobility density, either a function on \eqn{[0,1]} or a
numeric vector of values on \code{m_grid}.}

\item{m_grid}{Optional mobility grid used when \code{f} is numeric,
or to evaluate the integrals (default: uniform grid of 201 points).}

\item{tol}{Tolerance for \code{\link{uniroot}}.}
}
\value{
A list with \code{R_inf} (total final size),
  \code{MR_inf} (first moment), \code{R0} (basic reproduction
  number), and \code{converged} (logical).
}
\description{
Solves the scalar fixed-point equation
\deqn{M_R^\infty \;=\; \int_0^1 m f(m)\,(1 - e^{-\beta c M_R^\infty m})\,dm}
for the first mobility moment of the recovered compartment at the
end of the epidemic (with \eqn{I_0 \equiv 0}, \eqn{R_0 \equiv 0},
\eqn{S_0 = f}), and returns the total final epidemic size
\deqn{R_\infty \;=\; 1 - \int_0^1 f(m)\,e^{-\beta c M_R^\infty m}\,dm.}
When the basic reproduction number is less than or equal to one,
the function returns \eqn{R_\infty = 0}.
}
\examples{
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
f <- function(m) 6 * m * (1 - m)
final_size(pars, f)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/epi_quantities.R
\name{final_size_general}
\alias{final_size_general}
\title{Final epidemic size under general proportional initial conditions}
\usage{
final_size_general(params, f, m_grid = NULL, I_seed = 1e-04, tol = 1e-10)
}
\arguments{
\item{params}{A \code{"seair_params"} object.}

\item{f}{A mobility density, either a function on \eqn{[0,1]} or a
numeric vector of values on \code{m_grid}.}

\item{m_grid}{Optional mobility grid used when \code{f} is numeric,
or to evaluate the integrals (default: uniform grid of 201 points).}

\item{I_seed}{Initial symptomatic fraction.}

\item{tol}{Tolerance for \code{\link{uniroot}}.}
}
\value{
A list with \code{R_inf}, \code{MR_inf}, \code{R0}, and
  \code{converged}.
}
\description{
Solves the scalar equation of Theorem 4.3 (final-size relation),
\deqn{M_R^\infty + \int_0^1 m S_0(m)\,
    \exp\!\bigl(-\beta m [ c\,(M_R^\infty - M_R^0)
                        + (\delta/\gamma_I - c)\,M_I^0 ]\bigr) dm
    \;=\; \int_0^1 m f(m)\,dm,}
allowing a nonzero initial symptomatic fraction
\eqn{I_0 = I_{\mathrm{seed}} f}, with \eqn{E_0 = A_0 = R_0 = 0}.
}
\examples{
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
f <- function(m) 6 * m * (1 - m)
final_size_general(pars, f, I_seed = 0.001)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fit.R
\name{fit_mobility}
\alias{fit_mobility}
\title{Fit a mobility distribution to an observed symptomatic time series}
\usage{
fit_mobility(
  times,
  I_obs,
  params,
  K = 3,
  m_grid = seq(0, 1, length.out = 51),
  I_seed = 1e-04,
  start = NULL,
  n_restarts = 5,
  control = list(maxit = 500),
  verbose = FALSE
)
}
\arguments{
\item{times}{Numeric vector of observation times (starting at 0).}

\item{I_obs}{Observed aggregate symptomatic fraction at each time.}

\item{params}{A \code{"seair_params"} object holding fixed
transmission and stage parameters.}

\item{K}{Number of Beta-mixture components (default 3).}

\item{m_grid}{Mobility grid for the solver.}

\item{I_seed}{Initial symptomatic fraction in \eqn{[0,1)}.}

\item{start}{Optional finite user-supplied starting parameter vector of
length \eqn{3K - 1}.}

\item{n_restarts}{Number of random restarts (default 5).}

\item{control}{List of \code{\link[stats]{optim}} control options.}

\item{verbose}{Logical; if \code{TRUE}, print progress.}
}
\value{
A list with \code{weights}, \code{shape1}, \code{shape2}
  (the fitted Beta-mixture parameters), \code{f_hat} (density on
  \code{m_grid}), \code{m_grid}, \code{loss} (final SSE),
  \code{I_fit} (simulated aggregate symptomatic time series at
  \code{times}), and \code{optim_result}.
}
\description{
Fits a Beta-mixture mobility distribution \eqn{f} by minimising the
sum of squared errors between an observed aggregate symptomatic
time series \eqn{\langle I \rangle(t)} and the simulated output of
the mobility-based SEAIR model at the same times.  All transmission
and stage parameters are held fixed.
}
\details{
The unconstrained optimisation parameter is, for each component
\eqn{k = 1, \ldots, K}, a triple
\eqn{(\log a_k, \log b_k, \log \tilde{w}_k)} with mixture weights
recovered as \eqn{w_k = \tilde{w}_k / \sum_j \tilde{w}_j}.  The
first weight's log is fixed to 0 to remove the scale ambiguity,
so the parameter vector has length \eqn{3 K - 1}.
}
\examples{
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
m <- seq(0, 1, length.out = 21)
f_true <- dbeta(m, 3, 3)
init <- seair_init(m, f_true, I_seed = 1e-4)
times <- seq(0, 20, by = 5)
sol <- seair_solve(init, pars, times)
I_obs <- seair_aggregate(sol)$I

fit <- fit_mobility(times, I_obs, pars,
                    K = 1, m_grid = m,
                    start = log(c(3, 3)),
                    n_restarts = 0,
                    control = list(maxit = 20))
fit$loss

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solver.R
\name{seair_solve}
\alias{seair_solve}
\title{Solve the mobility-based SEAIR system}
\usage{
seair_solve(init, params, times, method = "lsoda", ...)
}
\arguments{
\item{init}{A \code{"seair_init"} object from \code{\link{seair_init}}.}

\item{params}{A \code{"seair_params"} object from
\code{\link{seair_params}}.}

\item{times}{Numeric vector of times at which the solution is
reported (must be increasing and include \code{0}).}

\item{method}{Integration method passed to \code{\link[deSolve]{ode}}
(default \code{"lsoda"}).}

\item{...}{Additional arguments passed to \code{\link[deSolve]{ode}}.}
}
\value{
A list of class \code{"seair_solution"} with entries
  \code{times}, \code{m_grid}, \code{S}, \code{E}, \code{A},
  \code{I}, \code{R} (each a matrix of dimension
  \code{length(times)} by \code{length(m_grid)}),
  together with the input \code{params} and \code{init}.
}
\description{
Discretises the mobility-based SEAIR system on the user-supplied
mobility grid (using the trapezoidal rule for the non-local
integral \eqn{\mathcal{K}}) and integrates the resulting coupled
ODE system over \code{times} with \pkg{deSolve}.
}
\examples{
m <- seq(0, 1, length.out = 31)
f <- dbeta(m, 2, 2)
init <- seair_init(m, f, I_seed = 1e-4)
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/params.R
\name{seair_params}
\alias{seair_params}
\title{Build a parameter list for the mobility-based SEAIR model}
\usage{
seair_params(beta, sigma, kappa, gamma_A, gamma_I, alpha = 1, delta = 1)
}
\arguments{
\item{beta}{Transmission rate \eqn{\beta > 0}.}

\item{sigma}{Rate \eqn{\sigma > 0} of progression from latent (E) to
asymptomatic (A).}

\item{kappa}{Symptom-onset rate \eqn{\kappa > 0} (A to I).}

\item{gamma_A}{Recovery rate \eqn{\gamma_A > 0} from the asymptomatic
compartment.}

\item{gamma_I}{Recovery rate \eqn{\gamma_I > 0} from the symptomatic
compartment.}

\item{alpha}{Relative infectiousness \eqn{\alpha \in [0,1]} of the
asymptomatic stage.}

\item{delta}{Mobility-reduction factor \eqn{\delta \in (0,1]} for the
symptomatic stage.}
}
\value{
A list of class \code{"seair_params"} containing the checked
  parameters.
}
\description{
Constructs and validates the list of transmission and stage-progression
parameters used by the mobility-based SEAIR model
\eqn{\partial_t S = -\beta m S \mathcal{K}},
\eqn{\partial_t E = \beta m S \mathcal{K} - \sigma E},
\eqn{\partial_t A = \sigma E - (\kappa + \gamma_A) A},
\eqn{\partial_t I = \kappa A - \gamma_I I},
\eqn{\partial_t R = \gamma_A A + \gamma_I I},
where
\eqn{\mathcal{K}(A,I) = \int_0^1 \bar{m}\,(\alpha A(\bar{m}) + \delta I(\bar{m}))\,d\bar{m}}.
}
\examples{
seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
             gamma_A = 0.1, gamma_I = 0.13,
             alpha = 0.5, delta = 0.3)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/solver.R
\name{seair_aggregate}
\alias{seair_aggregate}
\title{Aggregate a SEAIR solution over mobility}
\usage{
seair_aggregate(sol)
}
\arguments{
\item{sol}{A \code{"seair_solution"} object returned by
\code{\link{seair_solve}}.}
}
\value{
A data frame with columns \code{time}, \code{S}, \code{E},
  \code{A}, \code{I}, \code{R} (aggregate ratios).
}
\description{
Computes the aggregate time series
\eqn{\int_0^1 S(m,t)\,dm}, \eqn{\int_0^1 E(m,t)\,dm}, etc., using
the trapezoidal rule on the solver's mobility grid.
}
\examples{
m <- seq(0, 1, length.out = 31)
f <- dbeta(m, 2, 2)
init <- seair_init(m, f, I_seed = 1e-4)
pars <- seair_params(beta = 1.5, sigma = 0.3, kappa = 0.2,
                     gamma_A = 0.1, gamma_I = 0.13,
                     alpha = 0.5, delta = 0.3)
sol <- seair_solve(init, pars, times = seq(0, 30, by = 2))
agg <- seair_aggregate(sol)
head(agg)

}
